/**
*	An array implementation of a list of Cities. The array is of fixed size, set at construction.
*
*	@author D. Slater
*	@version 3/7/2003
*/

public class CityList {

	// instance variables
	private City [ ] list;
	private int count;
	
	/**
	*	Constructs an empty list with the specified initial capacity
	*
	*	@param int capacity - the initial capacity of the list
	*/
	public CityList(int capacity) {
	
		list = new City[capacity];
		count = 0;
	}
	
	/** 
	*	sorts list of cities by name, using a selection sort
	*/
	
	public void selectionSort() {
	
		for (int i = 0; i < size(); i++) {
		
			int index = findFirstAlphabetically(i);
			City temp = list[i];
			list[i] = list[index];
			list[index] = temp;
		}
	}
	
	private int findFirstAlphabetically(int pos) {
	
		int index = pos;
		for (int i = pos + 1; i < size(); i++) {
		
			if ((list[i].getName()).compareTo(list[index].getName()) < 0) {
				index = i;
			}
		}
		
		return index;
	}


	/**
	*	returns the index position of the target String (a city name), using Binary Search
	*	if it is located in the list; returns -1 if target not in the list
	*
	*	@param String target - the city name to be located
	*	@return int - the position of the target in the list if present; -1 otherwise
	*/
	public int binarySearch(String target) {
	
		int first = 0;
		int last = size() - 1;
		
		while (first <= last) {
		
			int midPt = (first + last) / 2;
			System.out.println("Probe at: " + list[midPt].getName());
			
			if ((list[midPt].getName()).equals(target)) {
				return midPt;
			} else if ((list[midPt].getName()).compareTo(target) < 0) {
				first = midPt + 1;
			} else {
				last = midPt - 1;
			}
		}
		
		return -1; 		// target not found
	}

	/**
	*	Appends the specified City to the end of this list
	*
	*	@param City city - thye City to be appended to this list
	*/
	public void add(City cty) {
	
		if (isFull()) {
			resize();
		} 
	
		list[count] = cty;
		count++;
	}
	

	/**
	*	Displays all of the elements of the list to the monitor
	*/
	public void printList() {
	
		for(int i = 0; i < count; i++) {
		
			System.out.println(list[i]);
		}
	}
	
	/**
	*	returns the average population of all the cities
	*
	*	@return double - the average population of all the cities in the list
	*/
	public double getAveragePop() {
	
		// if the list is empty, then average cannot be calculated
		if (isEmpty()) {
			return Double.NaN;		// a constant value used for this type of
									// error condition (NaN = "Not a number")
		}
		
		double total = 0.0;
		
		for (int i = 0; i < size(); i++) {
		
			total += list[i].getPopulation();
		}
		
		return total / size();
	}
	
	
	/**
	*	attempts to return the element at the position of the index in the list
	*
	*	@param int index - the position of the element
	*	@return City - the element at the index position, null if index > size()
	*/
	public  City get(int index) {
	
		// make sure the index value is valid
		if (index < size()) {
		
			return list[index];
		}
		
		// if the index is not valid, return null (a non-City, in this case)
		return null;
	}

	/**
	*	attempts to return the index of the element with the name that matches 
	*	the argument.
	*
	*	@param String cityName - the name we are looking for in the list
	*	@return int - the position of the found element, -1 if not found	
	*/
	public int findCity(String cityName) {
	
		// compare every element in the list with the target
		// the first match that is found, the index position of the element is returned
		for (int i = 0; i < size(); i++) {
		
			if (list[i].getName().equals(cityName)) {
				return i;
			}
		}
		// if a match is not found, -1 is not returned
		// (-1 is not a valid index position)
		return -1;
		
	}

	/**
	*	returns the city with the smallest population in the list
	*
	*	@return City - the city with the smallest population
	*/
	public City findSmallestPop() {
	
		// assume the first element is the smallest (what you are looking for
		City smallest = list[0];
		
		// compare all of the other elements in the list
		// if one is less, smallest is reset to the new element
		for (int i = 1; i < size(); i++) {		
			if (list[i].getPopulation() < smallest.getPopulation()) {
				smallest = list[i];
			}
		}		
		return smallest;
	}


	/**
	*	returns the city with the greatest population density in the list
	*
	*	@return City - the city with the greatest population density
	*/
	public City findGreatestPopDensity() {
	
		// assume the first element is the greatest (what you are looking for
		City greatest = list[0];
		
		// compare all of the other elements in the list
		// if one is greater, greatest is reset to the new element
		for (int i = 0; i < size(); i++) {
			if (list[i].getPopDensity() > greatest.getPopDensity()) {			
				greatest = list[i];
			}
		}
		return greatest;
	}

	/**
	*	prints all cities with area greater than the target
	*
	*	@param double target - used to determine the Cities to be printed
	*/
	public void printCitiesWithAreaGreaterThan(double target) {
	
		// filter the data
		int num = 0;
		
		for (int i = 0; i < size(); i++) {
		
			if (list[i].getArea() > target) {
			
				System.out.println(list[i].getName() + "   " + list[i].getArea());
				num++;
			}
		}
		
		// if none of the data matches the criteria
		if (num == 0) {
			System.out.println("No cities meet the criteria.");
		}
	}
	
	/**
	*	prints all cities with popDensity less than the target
	*
	*	@param double target - used to determine the Cities to be printed
	*/
	public void printCitiesWithPopDensityLessThan(double target) {
	
		// filter the data
		int num = 0;
		
		for (int i = 0; i < size(); i++) {
		
			if (list[i].getPopDensity() < target) {
			
				System.out.println(list[i].getName() + "   " + list[i].getPopDensity());
				num++;
			}
		}
		
		// if none of th data matches the criteria
		if (num == 0) {
			System.out.println("No cities meet the criteria.");
		}
	}
	
	/**
	*	prints all cities with population between the first and second value
	*	it is not assumed that first <= second
	*
	*	@param int first - used to determine the Cities to be printed
	*	@param int second - used to determine the Cities to be printed
	*/
	public void printCitiesWithPopBetween(int first, int second) {
	
		// determine order of the targets
		if (second < first) {
			int temp = first;
			first = second;
			second = temp;
		}
		
		// filter the data using the targets
		int num = 0;
		
		for (int i = 0; i < size(); i++) {
		
			if (list[i].getPopulation() > first && list[i].getPopulation() < second) {
			
				System.out.println(list[i].getName() + "   " + list[i].getPopulation());
				num++;
			}
		}
		
		// if none of the data matches the criteria
		if (num == 0) {
			System.out.println("No cities meet the criteria.");
		}
	}
	
	/**
	*	Returns the number of elements in this list. 
	*
	*	@return int - the number of elements in this list. 
	*/
	public int size() {
	
		return count;
	}
	
	/**
	*	Tests if this list has no elements. 
	*
	*	@return boolean - true if this list has no elements; false otherwise. 
	*/
	public boolean isEmpty() {
	
		return count == 0;
	}
	
	/**
	*	Returns the current number of slots available in this list. 
	*
	*	@return int - the current number of slots available in this list. 
	*/
	public int capacity() {
	
		return list.length;
	}
	
	/**
	*	Tests if this list has has room for no more elements. 
	*
	*	@return boolean - true if this list has no available slots; false otherwise. 
	*/	
	public boolean isFull() {
	
		return count == capacity();
	}
	
	/**
	*	Increases the capacity of this CityList instance, if  necessary, to ensure  
	*	that it can hold at least the double the current capacity of the list  
	* 
	*/
	private void resize() {
	
		City [] temp = new City [capacity() * 2 + 1];
		
		System.arraycopy(list, 0, temp, 0, list.length);
				
		list = temp;
	
	}
}